from fastapi import Request
from datetime import datetime
from fastapi.responses import JSONResponse
from starlette.middleware.base import BaseHTTPMiddleware


class LoggerMiddleware(BaseHTTPMiddleware):
    def __init__(self, app, enabled: bool = True):
        super().__init__(app)
        self.enabled = enabled

    async def dispatch(self, request: Request, call_next):
        if self.enabled:
            try:
                response = await call_next(request)
            except Exception as e:
                print(
                    f"\033[91m\033[1m\n [ {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} | {request.method} | 500 ][ {request.url.path} ] \033[0m"
                )
                print(f"\033[91m\033[1m\n [ Server Error ][ {repr(e)} ] \033[0m")
                return JSONResponse(
                    status_code=500,
                    content={
                        "code": 500,
                        "message": "There is a server exception. Please try to make the request again or contact [ ****** ] to report the exception.",
                        "path": request.url.path,
                        "data": {},
                    },
                )
            if (
                request.url.path == "/favicon.ico"
                or request.url.path == "/apple-touch-icon-precomposed.png"
                or request.url.path == "/apple-touch-icon.png"
            ):
                return JSONResponse(status_code=204, content=None)
            else:
                if 400 <= response.status_code < 500:
                    print(
                        f"\033[93m\033[1m\n [ {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} | {request.method} | {response.status_code} | Not Found ][ {request.url.path} ] \033[0m"
                    )
                    return JSONResponse(
                        status_code=response.status_code,
                        content={
                            "code": response.status_code,
                            "message": "Interface Not Found.",
                            "path": request.url.path,
                            "data": {},
                        },
                    )
                else:
                    print(
                        f"\033[92m\033[1m\n [ {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} | {request.method} | {response.status_code} | Successful ][ {request.url.path} ] \033[0m"
                    )
        return response
